﻿using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using UnivateProperties_API.Context;
using UnivateProperties_API.Model.Communication;

namespace UnivateProperties_API.Repository.Communication
{
    public class SMTPAccountRepository : IRepository<SMTPAccount>
    {
        private readonly DataContext _dbContext;

        public SMTPAccountRepository(DataContext dbContext)
        {
            _dbContext = dbContext;
        }

        public List<SMTPAccount> Get(Func<SMTPAccount, bool> where)
        {
            return _dbContext.Accounts.Where(where).ToList();
        }

        public List<SMTPAccount> GetAll()
        {
            return _dbContext.Accounts.ToList();
        }

        public SMTPAccount GetDetailed(Func<SMTPAccount, bool> first)
        {
            var item = _dbContext.Accounts.FirstOrDefault(first);

            if (item != null)
            {
                SMTPHostRepository host = new SMTPHostRepository(_dbContext);
                item.SMTPHost = host.GetDetailed(x => x.Id == item.SMTPHostId);
                return item;
            }
            else
            {
                throw(new Exception());
            }
        }

        public List<SMTPAccount> GetDetailedAll()
        {
            var list = GetAll();
            SMTPHostRepository host = new SMTPHostRepository(_dbContext);
            foreach (var item in list)
            {
                item.SMTPHost = host.GetDetailed(x => x.Id == item.SMTPHostId);
            }
            return list;
        }

        public void Insert(SMTPAccount item)
        {
            item.Id = NewId();
            _dbContext.Add(item);
            Save();
        }

        public void Insert(IEnumerable<SMTPAccount> items)
        {
            int id = NewId();
            foreach (var item in items)
            {
                item.Id = id;
                _dbContext.Add(item);
                id += 1;
            }
            Save();
        }

        public void Remove(SMTPAccount item)
        {
            var i = _dbContext.Accounts.Find(item);
            _dbContext.Accounts.Remove(i);
            Save();
        }

        public void Remove(IEnumerable<SMTPAccount> items)
        {
            foreach (var item in items)
            {
                SMTPAccount i = _dbContext.Accounts.Find(item);
                _dbContext.Accounts.Remove(i);
            }
            Save();
        }

        public void RemoveAtId(int item)
        {
            var i = _dbContext.Accounts.Find(item);
            _dbContext.Accounts.Remove(i);
            Save();
        }

        public void Update(SMTPAccount item)
        {
            _dbContext.Entry(item).State = EntityState.Modified;
            Save();
        }

        public void Save()
        {
            _dbContext.SaveChanges();
        }

        public int NewId()
        {
            int id = 0;
            if (_dbContext.Accounts.Count() > 0)
            {
                id = _dbContext.Accounts.Max(x => x.Id);
            }
            id += 1;
            return id;
        }
    }
}
